uniform sampler2DRect maskTex;
uniform vec2 maskSize;
uniform vec4 blends;
uniform float centerLuminance;
uniform float power;
uniform float gamma;
uniform vec4 normalizedBounds;  // start x, start y, end x, end y
uniform vec4 padding;           // left, bottom, right, top
varying vec2 vertex;

void main()
{
    vec2 pos = vec2( ( vertex.x - normalizedBounds.x ) / ( normalizedBounds.z - normalizedBounds.x ), ( vertex.y - normalizedBounds.y ) / ( normalizedBounds.w - normalizedBounds.y ) );
    if ( vertex.x < padding.x || vertex.y < padding.y || vertex.x > padding.z || vertex.y > padding.w || pos.x < 0.0 || pos.y < 0.0 || pos.x > 1.0 || pos.y > 1.0 )
    {
        gl_FragColor = vec4( 0.0, 0.0, 0.0, 1.0 );
    }
    else
    {
        float progress = 1.0;
        if ( pos.x < blends[3] )    // left
            progress = pos.x / blends[3];
        else if ( pos.x > 1.0 - blends[1] ) // right
            progress = (1.0 - pos.x) / ( blends[1] );
        if ( pos.y < blends[2] )
            progress *= pos.y / blends[2]; // bottom
        else if ( pos.y > 1.0 - blends[0] )
            progress *= (1.0 - pos.y) / ( blends[0] ); // top
        
        progress = progress < 0.5 ? centerLuminance * pow(2. * progress, power) : 1.0 - (1.0 - centerLuminance) * pow(2. * (1. - progress), power);
     
        vec4 mask;
        if ( maskSize.x > 0.0 )
            mask = texture2DRect( maskTex, vec2( gl_TexCoord[0].x * maskSize.x, ( 1.0 - gl_TexCoord[0].y ) * maskSize.y ) );
        else
            mask = vec4( 1.0, 1.0, 1.0, 1.0 );

        float blendAlpha;
        if ( gamma < 0.1 )
            blendAlpha = 1.0;
        else
            blendAlpha = pow(progress, 1. / gamma);
        gl_FragColor = vec4( gl_TexCoord[0].x, gl_TexCoord[0].y, blendAlpha * 0.3333 * mask.a * ( mask.r + mask.g + mask.b ), 1.0 );
    }
}